<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2006
sirfragalot@users.sourceforge.net

**********************************************************************************/
/*****
* BROWSECOMMON class
*
* Common methods for browsing as a 'tag cloud'
*
* 
$Header: /cvsroot/wikindx/wikindx3/core/browse/BROWSECOMMON.php,v 1.6 2006/04/29 00:44:46 sirfragalot Exp $
*****/
class BROWSECOMMON
{
// Constructor
	function BROWSECOMMON($db)
	{
		$this->db = $db;
		include_once("core/session/SESSION.php");
		$this->session = new SESSION();
		include_once("core/messages/MESSAGES.php");
		$this->messages = new MESSAGES();
		$this->start = $this->total = 0;
	}
/**
* userBibCondition() - return a SQL condition clause if we are browsing a user bibliography to ensure that 
* listed, selected or searched resources come only from that user bibliography. If we are browsing the 
* master WIKINDX bibliography, return FALSE
*
* @return	SQL condition clause or FALSE
*/
	function userBibCondition($table)
	{
		$condition = $usingBib = FALSE;
		if($useBib = $this->session->getVar("mywikindx_bibliography_use"))
		{
			$recordset = $this->db->select(array('WKX_user_bibliography'), array('bibliography'), 
				" WHERE " . $this->db->formatfield('id') . "=" . $this->db->tidyInput($useBib));
			$dbBibs = $this->db->fetchOne($recordset);
			$bibsArray = explode(",", $dbBibs);
			foreach($bibsArray as $resourceId)
				$conditionArray[] = $this->db->tidyInput($resourceId);
			$condition = " ($table.id = " . implode(" OR $table.id = ", $conditionArray) . ")";
			$recordset = $this->db->select(array('WKX_user_bibliography'), array('title'), 
				" WHERE " . $this->db->formatfield('id') . "=" . $this->db->tidyInput($useBib));
			$row = $this->db->fetchRow($recordset);
			$usingBib = stripslashes($row['title']);
		}
		return array($condition, $usingBib);
	}
// Work out text colour (shade of red) based on field frequency.
// Arbitrarily limit the red range to 50 steps.  Anything overflowing this has an increased font size with the maximum red.
	function colourText($lowestSum, $highestSum, $frequency)
	{
		$g = $b = 0;
		if($frequency == $lowestSum)
			return array($frequency, "#000000");
		else if($frequency < 20)
			$red = floor(($frequency/19) * 255);
		else
			$red = 255;
		if($red < 16)
			$r = '0' . dechex($red);
		else
			$r = dechex($red);
		$g = floor($g + ($red/10));
		$b = floor($b + ($red/10));
		if($g < 16)
			$g = '0' . dechex($g);
		else
			$g = dechex($g);
		if($b < 16)
			$b = '0' . dechex($b);
		else
			$b = dechex($b);
		$colour = "#" . $r . $g . $b;
//print "$colour<BR>";
		return array($red + $frequency, $colour);
	}
// Work out text size based on field frequency.
// Size range is 1em to 1.5em for items with $index of <= 255 and more for anything larger.
// Limit maximum size to 2.5;
	function sizeText($lowestSum, $highestSum, $index)
	{
		if($index > 255)
			$size = 1 + round($index/400, 2);
		else
			$size = 1 + round($index/512, 2);
		if($size > 2.5)
			$size = 2.5;
		return $size . "em";
	}
// Set the paging counter
	function setPaging($vars)
	{
		if(array_key_exists('pagingStart', $vars))
		{
			$this->session->setVar('mywikindx_pagingTagCloudStart', $vars['pagingStart']);
			$this->start = $vars['pagingStart'];
		}
		else
			$this->start = 0;
		$this->paging = $this->session->getVar("setup_pagingTagCloud");
		$this->maxLinks = $this->session->getVar('setup_pagingMaxLinks');
		$this->maxLinksHalf = round($this->maxLinks/2);
	}
// Return limited set from final array if paging required
	function paging($vars, $inputArray)
	{
		$this->setPaging($vars);
		if($this->paging <= 0) // unlimited
			return $inputArray;
		$this->total = sizeof($inputArray);
// NB - array_slice does not preserve keys until PHP  v5.0.2
		$keys = array_keys($inputArray);
		$values = array_values($inputArray);
		$keySlice = array_slice($keys, $this->start, $this->paging);
		$valueSlice = array_slice($values, $this->start, $this->paging);
// array_combine only for PHP v5
		foreach($keySlice as $key)
			$finalArray[$key] = array_shift($valueSlice);
		return $finalArray;
	}
	
// paging links if required
	function pagingLinks($queryString)
	{
		if(($this->paging <= 0) || ($this->total <= $this->paging))
			return FALSE;
		include_once("core/html/MISC.php");
		$end = $advanced = 0;
		$index = $maxLinks = 1;
		$advance = $this->start;
		while($advance >= (($this->maxLinksHalf * $this->paging) - $this->paging))
		{
			$end += $this->paging;
			$index += $this->paging;
			$advance -= $this->paging;
			$advanced++;
		}
		if($advanced)
		{
			$links[] = MISC::a("page", $this->messages->text("resources", "pagingStart"), 
				"index.php?" . htmlentities($queryString . "&pagingStart=0"));
			$maxLinks++;
		}
		while($index <= $this->total)
		{
			if($maxLinks++ >= $this->maxLinks)
				break;
			$end += $this->paging;
			if($end > $this->total)
				$end = $this->total;
			$start = $index - 1;
			$link = htmlentities($queryString . "&pagingStart=$start");
			$name = $index . " - " . $end;
			if($this->start == $start)
				$links[] = $name;
			else
				$links[] = MISC::a("page", $name, "index.php?" . $link);
			$index += $this->paging;
		}
		if($end < $this->total)
		{
			if($this->start && sizeof($links) == 1)
			{
				$links = array(MISC::a("page", $this->messages->text("resources", "pagingStart"), 
					"index.php?" . htmlentities($queryString . "&pagingStart=0")));
			}
			else if(sizeof($links) > 1)
			{
				$start = $this->total - ($this->total % $this->paging);
				if($start == $this->total)
					$start = $this->total - $this->paging;
				$links[] = MISC::a("page", $this->messages->text("resources", "pagingEnd"), 
					"index.php?" . htmlentities($queryString . "&pagingStart=$start"));
			}
		}
		return implode("&nbsp;&nbsp;&nbsp;|&nbsp;&nbsp;&nbsp;", $links);
	}
// Format display information string
	function linksInfo()
	{
		if($this->paging <= 0) // unlimited
			return FALSE;
		$displayEnd = $this->start + $this->paging;
		if(($this->paging <= 0) || ($displayEnd > $this->total))
			$displayEnd = $this->total;
		$displayStart = $this->start + 1;
		$bib = FALSE;
		return MISC::p($this->messages->text("hint", "pagingInfo", 
			" $displayStart - $displayEnd of " . $this->total . $bib));
	}
}
?>